<!DOCTYPE kpartgui>
<kpartgui name="rkward_console" version="730">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<Separator />
			<Action name="loadhistory"/>
			<Action name="savehistory"/>
			<Action name="file_print"/>
			<Separator />
			<Action name="file_export_html"/>
		</Menu>
		<Menu name="edit"><text>&amp;Edit</text>
<!-- Something strange is going on, here. When using the commented version, after switching
a few times between console and script editor, the Edit menu will start to become shuffled.
Probably a kdelibs bug (4.4.4).

The uncommented version is not quite correct (the "paste special" action will be misplaced),
but avoids the total derangement of the edit menu over time.
			<Action name="rkconsole_copy" group="edit_paste_merge"/>
			<Action name="rkconsole_copy_literal" group="edit_paste_merge"/>
			<Action name="rkconsole_paste" group="edit_paste_merge"/>
			<Separator/>
			<Action name="rkconsole_clear"/> -->
			<Action name="rkconsole_copy_literal"/>
			<Action name="rkconsole_copy_commands"/>
			<Action name="rkconsole_paste"/>
			<Separator/>
			<Action name="rkconsole_clear"/>
		</Menu>
		<Menu name="view"><text>&amp;View</text>
			<Action name="view_dynamic_word_wrap" />
			<Action name="view_show_minimap" />
			<Action name="view_inc_font_sizes" />
			<Action name="view_dec_font_sizes" />
		</Menu>
		<Menu name="run"><text>&amp;Run</text>
			<Action name="interrupt" group="postrun_actions_merge"/>
		</Menu>
	</MenuBar>
	<ToolBar fullWidth="true" name="mainToolBar">
		<Action name="interrupt" group="postrun_actions_merge"/>
	</ToolBar>
	<Menu name="rkconsole_context_menu">
		<Action name="rkconsole_copy_literal"/>
		<Action name="rkconsole_copy_commands"/>
		<DefineGroup name="rkconsole_context_merge_copy" />
		<Action name="rkconsole_paste"/>
		<DefineGroup name="rkconsole_context_merge_paste" />
		<Separator/>
		<DefineGroup name="rkconsole_context_merge1"/>
		<Action name="interrupt"/>
		<Separator/>
		<DefineGroup name="rkconsole_context_merge2"/>
		<Separator/>
		<Menu name="view"><text>&amp;View</text>
			<Action name="view_dynamic_word_wrap" />
			<Action name="view_show_minimap" />
			<Action name="view_inc_font_sizes" />
			<Action name="view_dec_font_sizes" />
		</Menu>
		<Separator/>
		<Action name="rkconsole_clear"/>
	</Menu>
</kpartgui>
