<document>
	<snippets>
		<snippet id="author_rkward_team">
			<author
				name="RKWard Team"
				email="rkward-devel@kde.org"
				url="https://rkward.kde.org" />
		</snippet>
		<snippet id="rkward_version_guard">
			<!-- Guard against accidental inclusion of pluginmaps in a different installed version of RKWard -->
			<dependencies rkward_min_version="0.7.5" rkward_max_version="0.7.5y" />
		</snippet>
	</snippets>
</document>
 
