<!DOCTYPE kpartgui>
<kpartgui name="rkward_commandlog" version="54">
	<MenuBar>
		<Menu name="edit"><text>&amp;Edit</text>
			<Action name="log_select_all"/>
			<Action name="log_copy"/>
			<Action name="log_clear"/>
		</Menu>
		<Menu name="settings"><text>&amp;Settings</text>
			<Action name="log_configure"/>
		</Menu>
	</MenuBar>
	<Menu name="rkcommandlog_context_menu">
		<Action name="log_copy"/>
		<DefineGroup name="rkcommandlog_context_merge1" append="rkcommandlog_context_merge1"/>
		<Separator/>
		<Action name="log_select_all"/>
		<Separator/>
		<Action name="log_clear"/>
		<Separator/>
		<Action name="log_configure"/>
	</Menu>
</kpartgui>
