<!DOCTYPE kpartgui>
<kpartgui name="rkward_main" version="742">
<MenuBar>
	<!-- The Main Window ui.rc is the only one, where merging happens, reliably. That is, why we need to define
	     a lot of merge points, here, which can then be used be mdi windows and their children.
	     That's a bit unfortunate, as it's rather non-modular, but so it is... -->
	<Menu name="file"><text>&amp;File</text>
		<Menu name="new_data"><text>&amp;New</text>
			<Action name="new_data_frame"/>
			<Action name="new_command_editor"/>
			<Action name="new_output"/>
		</Menu>
		<Action name="file_open_any"/>
		<Menu name="open_specific"><text>&amp;Open by Type</text>
			<Action name="file_open_script"/>
			<Action name="file_open_output"/>
		</Menu>
		<Action name="file_open_recenty"/>
		<Separator/>
		<Menu name="import"><text>&amp;Import</text>
			<Action name="import_data"/>
			<Separator/>
			<Merge/>
		</Menu>
		<Menu name="export"><text>&amp;Export</text></Menu>
		<Merge/>
		<Separator/>
		<Action name="file_quitx"/>
	</Menu>

	<Menu name="workspace"><text>&amp;Workspace</text>
		<Action name="file_openx"/>
		<Action name="file_open_recentx"/>
		<Action name="file_savex"/>
		<Action name="file_save_asx"/>
		<Separator/>
		<Action name="load_unload_libs"/>
	</Menu>

	<!-- These menus are actually defined in KParts. We also declare them here, to avoid menus jumping around -->
	<Menu name="edit"><text>&amp;Edit</text>
		<Action name="edit_menu_dummy"/>
		<DefineGroup name="edit_first_merge" />
		<Separator />
		<DefineGroup name="edit_undo_merge" />
		<Separator />
		<DefineGroup name="edit_paste_merge" />
		<DefineGroup name="after_edit_paste_merge" />
		<Separator />
		<DefineGroup name="edit_select_merge" />
		<Separator />
		<DefineGroup name="edit_find_merge" />
		<Separator />
	</Menu>
	<Menu name="view"><text>&amp;View</text>
		<Action name="view_menu_dummy"/>
	</Menu>
	<Menu name="run"><text>&amp;Run</text>
		<DefineGroup name="prerun_actions_merge" append="prerun_actions_merge"/>
		<DefineGroup name="run_actions_merge" append="run_actions_merge"/>
		<DefineGroup name="postrun_actions_merge" append="postrun_actions_merge"/>
		<Action name="cancel_all_commands"/>
		<Action name="restart_r"/>
		<Separator />
		<Action name="carbon_copy"/>
	</Menu>
	
	<Merge name="rkwardcomponents" />

	<Menu name="window"><text>&amp;Windows</text>
		<Merge/>
		<Action name="window_close"/>
		<Action name="window_close_all"/>
		<Action name="close_all_editors"/>
		<Separator/>
		<Action name="view_split_vert"/>
		<Action name="view_split_horiz"/>
		<Separator/>
		<Menu name="window_activate"><text>&amp;Activate</text>
			<Action name="left_window"/>
			<Action name="right_window"/>
			<Separator/>
			<Merge/>
		</Menu>
		<Separator/>
		<Action name="window_detach"/>
	</Menu>
	
	<Menu name="settings"><text>&amp;Settings</text>
		<DefineGroup name="configure_merge_group" append="configure_merge"/>
		<Separator append="configure_merge"/>
		<Action name="setup_wizard" append="configure_merge"/>
		<Action name="configure_packages" append="configure_merge"/>
		<Separator append="configure_merge"/>
	</Menu>

	<Menu name="help"><text>&amp;Help</text>
		<DefineGroup name="prehelp_actions_merge" append="prehelp_actions_merge"/>
		<DefineGroup name="help_actions_merge" append="help_actions_merge"/>
		<DefineGroup name="posthelp_actions_merge" append="posthelp_actions_merge"/>
	</Menu>
</MenuBar>
<ToolBar fullWidth="true" name="mainToolBar">
	<Action name="open_any"/>
	<Action name="new_any"/>
	<Action name="save_any"/>
	<Separator/>
	<Merge/>
	<DefineGroup name="pretoolbar_actions_merge" append="pretoolbar_actions_merge"/>
	<DefineGroup name="toolbar_actions_merge" append="toolbar_actions_merge"/>
	<DefineGroup name="posttoolbar_actions_merge" append="posttoolbar_actions_merge"/>
</ToolBar>
</kpartgui>
