<!DOCTYPE kpartgui>
<!-- This contains *mostly* a duplicate of rkwardui.rc. Perhaps the two can be merged one day, but that's not as trivial as it may sound.. -->
<kpartgui name="rkward_detachedwindow" version="54">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<DefineGroup name="file_dummy_merge" />
		</Menu>
		<Menu name="device" append="first_menu"><text>&amp;Device</text>
			<DefineGroup name="device_dummy_merge" />
		</Menu>
		<Menu name="edit"><text>&amp;Edit</text>
			<DefineGroup name="edit_first_merge" />
			<Separator />
			<DefineGroup name="edit_undo_merge" />
			<Separator />
			<DefineGroup name="edit_paste_merge" />
			<DefineGroup name="after_edit_paste_merge" />
			<Separator />
			<DefineGroup name="edit_select_merge" />
			<Separator />
			<DefineGroup name="edit_find_merge" />
			<Separator />
		</Menu>
		<Menu name="view"><text>&amp;View</text>
			<DefineGroup name="view_dummy_merge" />
		</Menu>
		<Menu name="history"><text>&amp;History</text>
			<DefineGroup name="history_dummy_merge" />
		</Menu>
		<Menu name="run"><text>&amp;Run</text>
			<DefineGroup name="prerun_actions_merge" append="prerun_actions_merge"/>
			<DefineGroup name="run_actions_merge" append="run_actions_merge"/>
			<DefineGroup name="postrun_actions_merge" append="postrun_actions_merge"/>
		</Menu>
		
		<Merge name="rkwardcomponents" />

		<Menu name="window"><text>&amp;Window</text>
			<Merge/>
			<Action name="dwindow_close"/>
			<Separator/>
			<Menu name="window_activate"><text>&amp;Activate</text></Menu>
			<Separator/>
			<Action name="dwindow_attach"/>
		</Menu>
		<Menu name="settings"><text>&amp;Settings</text>
			<DefineGroup name="settings_dummy_merge" />
		</Menu>
		<Menu name="help"><text>&amp;Help</text>
			<DefineGroup name="prehelp_actions_merge" append="prehelp_actions_merge"/>
			<DefineGroup name="help_actions_merge" append="help_actions_merge"/>
			<DefineGroup name="posthelp_actions_merge" append="posthelp_actions_merge"/>
		</Menu>
	</MenuBar>
</kpartgui>
