<!DOCTYPE kpartgui>
<kpartgui name="rkward_toplevel" version="741">
	<MenuBar>
		<Merge/>
		<Menu name="window"><text>&amp;Window</text>
			<Merge/>
			<Action name="output_show"/>
			<Action name="rkward_help"/>
			<Separator/>
			<Menu name="window_activate"><text>&amp;Activate</text>
				<Action name="prev_window"/>
				<Action name="next_window"/>
				<Separator/>
				<!-- NOTE: Actions for showing / hiding the various tool views are added below this placeholder, dynamically. Since some of those are plugins, their ids cannot be hardcoded, here. -->
				<Action name="window_show_PLACEHOLDER"/>
				<Separator/>
				<Action name="window_activate_docview"/>
			</Menu>
			<Merge/>
		</Menu>
		<Menu name="settings"><text>&amp;Settings</text>
			<Action name="colorscheme_menu" group="configure_merge_group"/>
			<Action name="switch_application_language" group="configure_merge_group"/>
			<Separator group="configure_merge_group"/>
			<Action name="options_configure_keybinding" group="configure_merge_group"/>
			<Action name="options_configure_toolbars" group="configure_merge_group"/>
		</Menu>
		<Menu name="help"><text>&amp;Help</text>
			<Action name="rkward_help" group="prehelp_actions_merge"/>
			<Action name="whats_this" group="prehelp_actions_merge"/>
			<Separator group="prehelp_actions_merge"/>
			<Action name="invoke_r_help" group="prehelp_actions_merge"/>

			<Action name="show_help_search" group="posthelp_actions_merge"/>
			<Separator group="posthelp_actions_merge"/>
			<Action name="report_bug" group="posthelp_actions_merge"/>
			<Separator group="posthelp_actions_merge"/>
			<Action name="about_app" group="posthelp_actions_merge"/>
		</Menu>
	</MenuBar>
</kpartgui>
