<!DOCTYPE kpartgui>
<kpartgui name="rkward_outputwindow" version="721">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<Action name="file_save"/>
			<Action name="file_save_as"/>
			<Action name="print_html"/>
			<Action name="save_html"/>
		</Menu>
		<Menu name="edit"><text>&amp;Edit</text>
			<Action name="copy"/>
			<Separator/>
			<Action name="find"/>
			<Action name="find_next"/>
			<Action name="find_previous"/>
			<Separator/>
			<Action name="select_all"/>
			<Separator/>
			<Action name="output_flush"/>
			<Action name="output_revert"/>
			<Action name="output_activate"/>
		</Menu>
		<Menu name="view"><text>&amp;View</text>
			<Action name="zoom_in"/>
			<Action name="zoom_out"/>
			<Action name="view_encoding"/>
			<Separator/>
			<Action name="output_refresh"/>
		</Menu>
	</MenuBar>
	<ToolBar fullWidth="true" name="mainToolBar">
		<Action name="output_revert"/>
		<Action name="output_refresh"/>
		<Action name="output_activate"/>
	</ToolBar>
</kpartgui>
